/* Example: function call by value and call by reference */

#include <stdio.h>

void swap1(int a, int b);      /* doesn't work */
void swap2(int *p1, int *p2);  /* works properly */

main()
{
  int x = 5, y = 6;
  
  printf("Initially:   x = %i, y = %i\n", x, y);
  
  swap1(x, y);  /* call by value */
  printf("After swap1: x = %i, y = %i\n", x, y);
  
  swap2(&x, &y);  /* call by reference */
  printf("After swap2: x = %i, y = %i\n", x, y);
}


/* Call by value: the function argument is of a simple data type. 
Its value is copied to the corresponding function parameter.
The copy may be changed, but on return the argument is 
unaffected. Used for an "input-only" argument. */

void swap1(int a, int b)  /* doesn't work */
{
  int temp;
  
  temp = a;  /* this swaps the values of parameters a and b, */
  a = b;     /* but doesn't change the arguments x and y in main */
  b = temp;
  
  return;
}


/* Call by reference: the function argument is a pointer (address of
a variable). The function has full access to the "pointee", and can 
change its value. Used for an "input/output" or "output only" 
argument.*/

void swap2(int *p1, int *p2)  /* works properly */
{
  int temp;
  
  temp = *p1;  /* this swaps the values of "pointees" *p1 and *p2, */
  *p1 = *p2;   /* so changes x and y in main, as desired */
  *p2 = temp;
  
  return;
}
